//nolint:dupl
package iotago

import (
	"cmp"

	"github.com/iotaledger/hive.go/serializer/v2"
)

// AddressUnlockCondition is an UnlockCondition defining an address which has to be unlocked.
type AddressUnlockCondition struct {
	Address Address `serix:""`
}

func (s *AddressUnlockCondition) Clone() UnlockCondition {
	return &AddressUnlockCondition{Address: s.Address.Clone()}
}

func (s *AddressUnlockCondition) StorageScore(storageScoreStruct *StorageScoreStructure, _ StorageScoreFunc) StorageScore {
	return s.Address.StorageScore(storageScoreStruct, nil)
}

func (s *AddressUnlockCondition) WorkScore(_ *WorkScoreParameters) (WorkScore, error) {
	// AddressUnlockCondition does not require a signature check on creation, only consumption.
	return 0, nil
}

func (s *AddressUnlockCondition) Compare(other UnlockCondition) int {
	return cmp.Compare(s.Type(), other.Type())
}

func (s *AddressUnlockCondition) Equal(other UnlockCondition) bool {
	otherUnlockCond, is := other.(*AddressUnlockCondition)
	if !is {
		return false
	}

	return s.Address.Equal(otherUnlockCond.Address)
}

func (s *AddressUnlockCondition) Type() UnlockConditionType {
	return UnlockConditionAddress
}

func (s *AddressUnlockCondition) Size() int {
	// UnlockType + Address
	return serializer.SmallTypeDenotationByteSize + s.Address.Size()
}
